@@ -12,8 +12,8 @@ nanotesterr <- function(x, e = "")
   invisible(grepl(e, tryCatch(x, error = identity)[["message"]], fixed = TRUE) || stop("expected error message '", e, "' not generated"))
 
 nng_version()
-nanotestnano(n <- nano("pair", listen = "inproc://nanonext", autostart = FALSE))
-nanotestnano(n1 <- nano("pair", dial = "inproc://nanonext", autostart = FALSE))
+nanotestnano(n <- nano("req", listen = "inproc://nanonext", autostart = FALSE))
+nanotestnano(n1 <- nano("rep", dial = "inproc://nanonext", autostart = FALSE))
 nanotest(inherits(n, "nanoObject"))
 nanotest(inherits(n$socket, "nanoSocket"))
 nanotest(inherits(n$socket, "nano"))
@@ -22,12 +22,13 @@ n$newmethod <- "doesnotwork"
 nanotestn(n$newmethod)
 nanotest(is.integer(attr(n$socket, "id")))
 nanotest(n$socket$state == "opened")
-nanotest(n$socket$protocol == "pair")
-nanotest(n$send("not ready") == 8L)
-nanotest(n$recv() == 8L)
+nanotest(n$socket$protocol == "req")
+nanotest(n$send("not ready", mode = "next") == 8L)
+nanotest(n$recv() == 11L)
 nanotestnano(n$opt("recv-size-max", 8192))
 nanotest(n$opt("recv-size-max") == 8192L)
 nanotestnano(n$opt("recv-buffer", 8L))
+nanotestnano(n$opt("req:resend-time", 0L))
 nanotestnano(n$opt("socket-name", "nano"))
 nanotest(n$opt("socket-name") == "nano")
 nanotesterr(n$opt("socket-name", NULL), "argument")
@@ -65,23 +66,24 @@ nanotest(n1$dialer[[1]]$state == "started")
 nanotesterr(n$send(list(), mode = "raw"), "atomic vector type")
 nanotesterr(n$recv(mode = "none"), "mode")
 nanotesterr(n$recv(mode = "c"), "mode")
-nanotestaio(raio <- n$recv_aio(timeout = 1L))
+nanotestaio(raio <- n1$recv_aio(timeout = 1L))
 nanotestp(raio)
 nanotest(is_error_value(call_aio(raio)$data))
 nanotest(is_error_value(raio$data))
 nanotestz(n$send(data.frame(), block = FALSE))
-nanotestnn(n1$recv(block = FALSE))
+nanotest(is.data.frame(n1$recv(block = 500)))
 nanotestz(n1$send(c("test", "", "spec"), mode = "raw", block = 500))
 nanotest(identical(n$recv("character", block = 500), c("test", "", "spec")))
-nanotestz(n1$send(c(1L, 0L, 0L), mode = "r"))
-nanotest(is.integer(n$recv("int")))
-nanotestaio(saio <- n$send_aio(data.frame(random(1e3L), mode = 3L), timeout = 900))
+nanotestz(n$send(c(1L, 0L, 0L), mode = "r"))
+nanotest(length(n1$recv("int", block = 500)) == 3L)
+nanotestaio(saio <- n1$send_aio(random(1e3L), mode = 3L, timeout = 900))
 nanotestp(saio)
 nanotestaio(call_aio(saio))
-nanotest(is.integer(saio$result))
+nanotestz(saio$result)
 nanotesterr(n$send("wrong mode", mode = "none"), "mode")
-nanotestaio(raio <- n1$recv_aio(timeout = 500))
+nanotestaio(raio <- n$recv_aio(timeout = 500))
 nanotestp(raio)
+nanotest(nchar(call_aio(raio)[["value"]]) == 2000L)
 raio$newfield <- "doesnotwork"
 raio[["newfield"]] <- "doesnotwork"
 nanotestn(raio$newfield)
@@ -132,16 +134,10 @@ nanotestw(is.raw(call_aio(rraio)$data))
 nanotestaio(sraio <- n$send_aio(as.raw(0L), mode = "raw", timeout = 500))
 nanotestaio(rraio <- n1$recv_aio(mode = "complex", timeout = 500))
 nanotestw(is.raw(call_aio(rraio)$data))
-nanotestaio(sraio <- n$send_aio("hello", mode = 2L, timeout = 500))
-assign("aio", value = "test", envir = sraio)
-nanotesterr(sraio$result, "valid")
-nanotestaio(rraio <- n1$recv_aio(mode = "charac", timeout = 500))
 nanotesterr(opt(rraio[["aio"]], "false") <- 0L, "valid")
 nanotesterr(subscribe(rraio[["aio"]], "false"), "valid")
 nanotesterr(opt(rraio[["aio"]], "false"), "valid")
 nanotesterr(stat(rraio[["aio"]], "pipes"), "valid")
-assign("aio", value = "test", envir = rraio)
-nanotesterr(rraio$data, "valid")
 
 nanotestz(n$dial(url = "inproc://two", autostart = FALSE))
 nanotestz(n$dialer_start())
@@ -206,23 +202,32 @@ nanotest(!unresolved(msg))
 nanotest(is.integer(req$send(c(TRUE, FALSE, TRUE), mode = 2L, block = 500)))
 nanotestaio(msg <- recv_aio(ctx, mode = 6L, timeout = 500))
 nanotest(is.logical(call_aio(msg)$data))
-nanotestaio(err <- send_aio(ctx, msg, mode = "raw"))
+nanotestaio(err <- send_aio(ctx, msg[["data"]], mode = "next"))
 nanotestn(stop_aio(err))
 nanotesterr(err$result, "active")
 nanotestaio(err <- send_aio(ctx, "test"))
 nanotest(is_error_value(call_aio(err)$result))
 nanotest(is.integer(req$send(as.raw(1L), mode = 2L, block = 500)))
 nanotestaio(msg <- call_aio(recv_aio(ctx, mode = 8L, timeout = 500)))
-nanotestaio(envraw <- send_aio(ctx, msg, mode = 2L, timeout = 500))
+nanotestaio(saio <- send_aio(ctx, msg, mode = 2L, timeout = 500))
 nanotest(identical(req$recv(mode = 8L, block = 500), as.raw(1L)))
 nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = 2L, recv_mode = "complex", timeout = 500))
 nanotest(is.integer(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500)))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 
+nanotest(is.pairlist(nxt <- nextmode(inhook = function(x) serialize(x, NULL), outhook = unserialize)))
+nanotest(length(nxt) == 2L)
+nanotest(is.function(nxt[[1L]]))
+nanotest(is.integer(req$send(list(new.env(), new.env()), mode = 3L, block = 500)))
+nanotest(is.environment(recv(rep, block = 500)[[1L]]))
+nanotest(is.pairlist(nextmode(NULL, NULL, mark = TRUE)))
+nanotestn(unlist(nextmode()))
+
 nanotest(inherits(cv <- cv(), "conditionVariable"))
 nanotestp(cv)
+nanotest(!.until(cv, 10L))
+nanotest(!.until(cv, 10))
 nanotest(until(cv, 10L))
-nanotest(until(cv, 10))
 nanotestz(cv_reset(cv))
 nanotestz(cv_value(cv))
 nanotestaio(cs <- request_signal(req$context, "test", send_mode = "next", cv = cv, timeout = 500))
@@ -232,22 +237,18 @@ nanotest(call_aio(cr)$data == "test")
 nanotest(is.integer(send(ctxn, TRUE, mode = 0L, block = FALSE)))
 nanotest(is.integer(recv(ctxn, mode = 8L, block = FALSE)))
 nanotest(typeof(ctxn <- .context(rep)) == "externalptr")
-nanotestaio(cs <- request(.context(req$socket), data = TRUE, send_mode = "raw", ack = TRUE))
-nanotest(recv(ctxn, mode = "logical", block = 500))
-nanotestz(send(ctxn, TRUE, mode = 0L, block = 500))
-nanotestz(cv_reset(cv))
-nanotestaio(cr <- recv_aio_signal(rep, cv = cv, mode = 8L))
-nanotest(wait(cv))
+nanotestaio(cs <- request(.context(req$socket), data = TRUE))
+nanotest(recv(ctxn, block = 500))
+nanotestz(send(ctxn, TRUE, mode = 3L, block = 500))
 nanotestz(reap(ctxn))
-nanotestz(length(.subset2(cr, "data")))
-nanotestz(pipe_notify(rep, cv))
+nanotestz(pipe_notify(rep, cv, add = TRUE, remove = TRUE, flag = TRUE))
 nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
 nanotesterr(request_signal(err, "test", cv = cv), "valid")
 nanotesterr(recv_aio_signal(err, cv = cv, timeout = 500))
 nanotesterr(request_signal(.context(req$socket), "test", cv = err), "valid")
 nanotesterr(recv_aio_signal(rep, "test", cv = err), "valid")
 nanotesterr(wait(err), "valid")
-nanotesterr(until(err, 10), "valid")
+nanotesterr(.until(err, 10), "valid")
 nanotesterr(cv_value(err), "valid")
 nanotesterr(cv_reset(err), "valid")
 nanotesterr(cv_signal(err), "valid")
@@ -267,7 +268,7 @@ nanotestnano(pub <- nano("pub", listen = "inproc://ps"))
 nanotestnano(sub <- nano("sub", dial = "inproc://ps", autostart = NA))
 nanotestxp(cv <- cv())
 nanotestxp(cv2 <- cv())
-nanotestz(pipe_notify(pub$socket, cv, cv2, flag = FALSE))
+nanotestz(pipe_notify(pub$socket, cv, cv2, add = TRUE, remove = TRUE, flag = FALSE))
 nanotestnano(sub$opt(name = "sub:prefnew", value = FALSE))
 nanotest(!sub$opt(name = "sub:prefnew"))
 nanotesterr(sub$opt(name = "false", value = 100), "supported")
@@ -293,7 +294,7 @@ nanotest(cv_value(cv) == 1L)
 nanotestnano(surv <- nano(protocol = "surveyor", listen = "inproc://sock1", dial = "inproc://sock2"))
 nanotestp(surv)
 nanotestnano(resp <- nano(protocol = "respondent", listen = "inproc://sock2", dial = "inproc://sock1"))
-nanotestz(pipe_notify(surv$socket, cv, cv2, flag = TRUE))
+nanotestz(pipe_notify(surv$socket, cv, cv2, add = TRUE, remove = TRUE, flag = TRUE))
 surv$dialer <- NULL
 nanotestxp(surv$dialer[[1L]])
 nanotestxp(surv$listener[[1L]])
@@ -312,17 +313,17 @@ nanotest(is_error_value(resp$recv()))
 nanotest(inherits(bus <- socket(protocol = "bus"), "nanoSocket"))
 nanotest(inherits(push <- socket(protocol = "push"), "nanoSocket"))
 nanotest(inherits(pull <- socket(protocol = "pull"), "nanoSocket"))
-nanotest(inherits(survey <- nano(protocol = "surveyor"), "nanoObject"))
-nanotest(inherits(respond <- nano(protocol = "respondent"), "nanoObject"))
+nanotest(inherits(pair <- socket(protocol = "pair"), "nanoSocket"))
 nanotestnano(bus)
 nanotestw(listen(bus, url = "test") == 3L)
 nanotestw(dial(bus, url = "test") == 3L)
+nanotesterr(listen(bus, url = "tls+tcp://localhost/:0", tls = "wrong"), "valid TLS")
+nanotesterr(dial(bus, url = "tls+tcp://localhost/:0", tls = "wrong"), "valid TLS")
 nanotestz(close(bus))
 nanotestw(close(bus) == 7L)
 nanotestz(close(push))
 nanotestz(reap(pull))
-nanotestz(survey$close())
-nanotestz(respond$close())
+nanotestz(reap(pair))
 nanotesterr(socket(protocol = "newprotocol"), "protocol")
 nanotesterr(socket(dial = "test"), "argument")
 nanotesterr(socket(listen = "test"), "argument")
@@ -333,10 +334,10 @@ nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = list("date"
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = TRUE))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000))
-relo <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = "server")
-nanotest(is.integer(call_aio(relo)$status))
-relo <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = list("server"))
-nanotest(is.integer(call_aio(relo)$status))
+haio <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = "server")
+nanotest(is.integer(call_aio(haio)$status))
+haio <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = list("server"))
+nanotest(is.integer(call_aio(haio)$status))
 put1 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("date", "server"), timeout = 3000L)
 nanotest(is.integer(call_aio(put1)$status))
 nanotestnn(put1$headers)
@@ -345,8 +346,8 @@ put2 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = list(
 nanotestp(put2)
 nanotest(is.integer(call_aio(put2)$status))
 nanotest(is.null(stop_aio(put2)))
-nanotest(is_error_value(ncurl("http")$data))
-nanotest(is_error_value(ncurl_aio("http")$data))
+haio <- ncurl("http")
+nanotest(is_error_value(haio$data))
 haio <- ncurl_aio("https://i.i")
 nanotest(is_error_value(call_aio(haio)$data))
 nanotestp(haio$data)
@@ -355,14 +356,18 @@ nanotestw(is.raw(call_aio(ncaio)$data) || is_error_value(ncaio$data))
 sess <- ncurl_session("https://postman-echo.com/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 3L)
+nanotest(is_error_value(sess) || close(sess) == 0L)
 sess <- ncurl_session("https://postman-echo.com/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 3L)
 nanotest(is_error_value(sess) || close(sess) == 0L)
 nanotesterr(transact(sess), "ncurlSession")
 nanotestw(is_error_value(ncurl_session("https://i")))
+nanotesterr(ncurl("https://www.cam.ac.uk/", tls = "wrong"), "valid TLS")
 nanotesterr(stream(dial = "wss://127.0.0.1:5555"))
+nanotesterr(stream(dial = "wss://127.0.0.1:5555", tls = "wrong"), "valid TLS")
 nanotesterr(stream(listen = "errorValue3"), "argument")
+nanotesterr(stream(listen = "errorValue3", tls = "wrong"), "valid TLS")
 nanotesterr(stream(), "specify")
 
 nanotest(is.character(ver <- nng_version()) && length(ver) == 2L)
@@ -387,6 +392,7 @@ for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511))
   nanotest(is.character(status_code(i)))
 nanotest(strcat("hello ", "world") == "hello world")
 
+nanotestxp(cv <- cv())
 s <- tryCatch(stream(dial = "wss://echo.websocket.events/", textframes = TRUE), error = function(e) NULL)
 if (length(s)) {
   nanotestnn(recv(s, block = 500L))
@@ -407,7 +413,7 @@ if (length(s)) {
   nanotestaio(sr <- recv_aio_signal(s, mode = "double", timeout = 500L, cv = cv))
   nanotestw(length(call_aio(sr)[["data"]]))
   nanotest(cv_value(cv) > 0L)
-  nanotest(!wait(cv))
+  nanotest(wait(cv))
   nanotest(is.character(opt(s, "ws:request-headers")))
   nanotestnn(opt(s, "tcp-nodelay") <- FALSE)
   nanotesterr(recv(s, mode = "none", block = FALSE), "mode")
@@ -425,23 +431,18 @@ nanotestaio(send_aio(s, "test"))
 nanotestnn(recv(s1, block = 500))
 nanotest(is_error_value(recv(s2)))
 nanotestz(unlock(s))
-nanotesterr(unlock(cv), "valid Socket")
-nanotestxp(cv <- cv())
-nanotestz(pipe_notify(s, cv = cv, flag = FALSE))
+nanotestz(pipe_notify(s, cv = cv, add = TRUE, remove = TRUE))
+nanotestz(lock(s, cv = cv))
 nanotestnano(s3 <- socket("bus", dial = "inproc://nanolock"))
 nanotestz(send(s, "test", block = 500))
 nanotestnn(recv(s3, block = 500))
-nanotestz(lock(s, cv = cv))
+nanotesterr(unlock(cv), "valid Socket")
 nanotesterr(lock(cv), "valid Socket")
 nanotesterr(lock(s, cv = s), "valid Condition Variable")
-nanotestnano(s4 <- socket("bus", dial = "inproc://nanolock"))
-nanotestaio(send_aio(s, "test"))
-nanotest(is_error_value(recv(s4)))
 nanotestz(close(s))
 nanotestz(close(s1))
 nanotestz(close(s2))
 nanotestz(close(s3))
-nanotestz(close(s4))
 
 nanotest(nanonext:::.DollarNames.ncurlAio(NULL, "sta") == "status")
 nanotest(nanonext:::.DollarNames.recvAio(NULL, "dat") == "data")
@@ -459,6 +460,7 @@ nanotesterr(stat(fakesock, "pipes"), "valid Socket")
 nanotesterr(close(fakesock), "valid Socket")
 nanotest(!.unresolved(fakesock))
 fakectx <- `class<-`("test", "nanoContext")
+nanotest(!unresolved(fakectx))
 nanotest(!.unresolved(fakectx))
 nanotesterr(request(fakectx, data = "test"), "valid Context")
 nanotesterr(subscribe(fakectx, NULL), "valid")
@@ -532,7 +534,8 @@ nanotestxp(tls <- tls_config(client = cert$client))
 nanotest(inherits(tls, "tlsConfig"))
 nanotestp(tls)
 nanotest(is_error_value(ncurl("https://www.cam.ac.uk/", tls = tls)$status))
-nanotest(is_error_value(call_aio(ncurl_aio("https://www.cam.ac.uk/", tls = tls))$status))
+haio <- ncurl_aio("https://www.cam.ac.uk/", tls = tls)
+nanotest(is_error_value(call_aio(haio)$status))
 nanotesterr(ncurl_session("https://www.cam.ac.uk/", tls = cert$client), "not a valid TLS")
 nanotestw(typeof(b <- ncurl_session("https://www.cam.ac.uk/", tls = tls)) == "externalptr" || is_error_value(b))
 nanotest(if (!is_error_value(b)) is_error_value(transact(b)[["headers"]]) else TRUE)
